/*
  mainmenu.cpp
  Copyright (C) 2008-2010 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "qgmap.h"
#include "mainmenu.h"
#include <qpopupmenu.h>
#include "gpsmenu.h"
#include "qgpopupmenu.h"

#define ADDITEM(str, evno) insertItem(QString::fromUtf8(str), evno)

MainMenu::MainMenu(QWidget *parent, const char *name) :
	QMenuBar(parent, name)
{
	settings_menu = new QGPopupMenu(this, "settingsmenu");
	settings_menu->ADDITEM("地図画像のパス設定", EV_SET_MAP_PATH);
	settings_menu->insertItem(QString::fromUtf8("GPS設定"),
							  new GPSMenu(this, "gps_submenu"), EV_GPS_MENU);

	insertItem(QString::fromUtf8("設定"), settings_menu, EV_SETTINGS_MENU);

	QPopupMenu *view_menu = new QGPopupMenu(this, "viewmenu");
	view_menu->ADDITEM("Zoom IN (詳細) [I]", EV_ZOOM_IN);
	view_menu->ADDITEM("Zoom OUT (広域) [O]", EV_ZOOM_OUT);
	view_menu->insertSeparator();
	view_menu->ADDITEM("センターマーカー [.]", EV_TOGGLE_CENTER_MARKER);
	view_menu->ADDITEM("座標表示 [Z]", EV_TOGGLE_GRID);
	view_menu->ADDITEM("ガンマ補正", EV_SET_GAMMA);

	QPopupMenu *marker_menu = new QGPopupMenu(this, "markermenu");
	marker_menu->ADDITEM("表示 [M]", EV_TOGGLE_MARKER);
	marker_menu->insertSeparator();
	for (int i = 1; i <= 10; i++)
		marker_menu->insertItem(QString::fromUtf8("マーカー%1").arg(i % 10),
								EV_TOGGLE_MARKER_0 + (i % 10));

	view_menu->insertItem(QString::fromUtf8("マーカー"), marker_menu, EV_DISP_MARKER_MENU);

	insertItem(QString::fromUtf8("表示"), view_menu, EV_DISP_MENU);

	QPopupMenu *location_menu = new QGPopupMenu(this, "locationmenu");
	location_menu->ADDITEM("PHS 位置取得 [P]", EV_GET_PHS_POS);
	location_menu->ADDITEM("GPS 位置取得 [G]", EV_TOGGLE_GPS);
	location_menu->ADDITEM("緯度経度指定ジャンプ [J]", EV_JUMP);
	location_menu->ADDITEM("現在地をコピー [Fn+C]", EV_COPY_LOCATION);
	location_menu->ADDITEM("現在地をペースト [Fn+V]", EV_PASTE_LOCATION);

	QPopupMenu *marker_jump_menu = new QGPopupMenu(this, "marker_jumpmenu");
	for (int i = 1; i <= 10; i++)
		marker_jump_menu->insertItem(QString::fromUtf8("マーカー%1 [%2]").arg(i % 10).arg(i % 10),
									EV_JUMP_MARKER_0 + (i % 10));

	location_menu->insertItem(QString::fromUtf8("マーカージャンプ"), marker_jump_menu, EV_JUMP_MARKER_MENU);

	QPopupMenu *marker_save_menu = new QGPopupMenu(this, "marker_savemenu");
	for (int i = 1; i <= 10; i++)
		marker_save_menu->insertItem(QString::fromUtf8("マーカー%1 [S+%2]").arg(i % 10).arg(i % 10),
									EV_SAVE_MARKER_0 + (i % 10));

	location_menu->insertItem(QString::fromUtf8("マーカー登録"), marker_save_menu, EV_SAVE_MARKER_MENU);

	insertItem(QString::fromUtf8("位置"), location_menu, EV_LOCATION_MENU);

	QPopupMenu *poi_menu = new QGPopupMenu(this, "poimenu");
	poi_menu->ADDITEM("POI管理/表示切替", EV_POI_MANAGE);
	poi_menu->ADDITEM("POI検索", EV_POI_SEARCH);

	insertItem(QString::fromUtf8("POI"), poi_menu, EV_POI_MENU);
}

QPopupMenu *MainMenu::create_submenu(const char *name,
									 struct menu_items *items, int count)
{
	QPopupMenu *submenu = new QGPopupMenu(this, name);

	if (!submenu)
		return NULL;

	while (count-- > 0) {
		submenu->insertItem(QString::fromUtf8(items->name), items->id, 0);
		items++;
	}

	return submenu;
}

void MainMenu::keyPressEvent(QKeyEvent * e)
{
	QMenuBar::keyPressEvent(e);

	if (e->key() == Key_Escape) {
		parentWidget()->setFocus();
	}
}
