/*
  mainwidget.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MAINWIDGET_H__
#define __MAINWIDGET_H__

#include "layeredbase.h"
#include "qgconf.h"
#include "qobjectlist.h"
#include <qpixmap.h>
#include <qstring.h>

class QWidget;
class QMenuBar;
class QPopupMenu;
class Pos;

class Help;
class Info;
class IconMenu;
class CenterPos;
class PoiInfoView;

class Mounter;

#ifndef QWS
class QLabel;
#endif

class MainWidget : public LayeredBase
{
	Q_OBJECT
  public:
	MainWidget( QWidget *parent =0, const char *name = 0, WFlags fl = 0);
	~MainWidget();

	void dispInfo(QString str);
	void hideInfo();

	void dispPoiInfo(QPixmap const &icon, QString const &title,
					 QString const &desc, QString const &info);

	void mousePressEvent(QMouseEvent *e);
	void keyPressEvent (QKeyEvent * e);
	void paintEvent(QPaintEvent *e);

	const Pos &getPosition(void);

  private:
	LayeredWidget *gmap;
	LayeredWidget *slider;
	LayeredWidget *marker;
	LayeredWidget *menu;
	LayeredWidget *scale;
	LayeredWidget *gpsview;
	LayeredWidget *poiview;
	LayeredWidget *centermarker;
	LayeredWidget *freehand;

	Help *help;
	Info *info;
	PoiInfoView *poiinfo;

	CenterPos *center;

#ifndef QWS
	QLabel *label;
#endif

  signals:
	void zoomChanged(int value);
	void setPosition(Pos &pos);
	void positionChanged(Pos &pos);
	void configChanged(const QString &name, const QString &value);
	void reload(bool);
	void cacheFlush();
	void updateMap();
	void movePosition(Pos &pos);
};

#endif // __MAINWIDGET_H__
