/*
  poi_db.h
  Copyright (C) 2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __POI_DB_H__
#define __POI_DB_H__

#include <qstring.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qarray.h>
#include "sqlite.h"
#include <sqlite3.h>

class PoiDbItem;
class PoiDbFile;
class PoiDbIcon;

class PoiDb
{
  public:
	PoiDb(const char *db_file);
	~PoiDb();

	PoiDbItem *queryPoiMapItem(int x, int y, int zoom, int limit, bool *too_many = NULL);
	PoiDbItem *queryPoiItem(int x, int y, int zoom);
	PoiDbItem *queryPoiItem(int id);
	PoiDbFile *queryPoiFile(int grp_id, int id = -1);
	PoiDbIcon *queryPoiIcon();
	int importPoiCSV(QString file_path, int icon_id = 0, int zoom = 7, QString desc = "",
					 void (*callback)(int now, int total, void *data) = NULL, void *data = NULL);
	int addIcon(const QByteArray &data, int icon_id = -1);
	int delIcon(int id);
	int delFile(int id);

	int countItem(const QString keyword, const QArray<int> &file_id = 0);
	int countItem(const QString keyword, int x, int y, int radius,
				  const QArray<int> &file_id = 0);
	PoiDbItem *searchItem(const QString keyword, const QArray<int> &file_id = 0);
	PoiDbItem *searchItem(const QString keyword, int x, int y, int radius,
						  const QArray<int> &file_id = 0);

	static QStringList splitCSV(const QString str);

	QByteArray getPoiIcon(int id);

	int changePoiFile(int id, int icon_id, int zoom, QString name);

	int changePoiFileEnable(int id, int enable);

	int beginTransaction();
	int commitTransaction();

	enum {
		PRESET_OFFSET = 10000,
	};

  private:
	SQLite *db;

	int getNewFileId();
	int getNewIconId();

	QString createIdQuery(const QString name, const QArray<int> &ids) const;
	static void rough_distance(sqlite3_context *context, int argc, sqlite3_value **argv);
};

class PoiDbResult
{
  public:
	PoiDbResult(SQLiteStatement *s);
	~PoiDbResult();

	bool is_end();
	bool next();
	int count();

  protected:
	SQLiteStatement *stmt;

  private:
	int status;
};

class PoiDbItem : public PoiDbResult
{
  public:
	PoiDbItem(SQLiteStatement *s, int x, int y, int zoom);

	int getId();
	int getX();
	int getY();
	QString getName();
	QString getInfo();
	int getIconId();
	int getFileId();

  private:
	int map_x;
	int map_y;
	int map_zoom;
};

class PoiDbFile : public PoiDbResult
{
  public:
	PoiDbFile(SQLiteStatement *s);

	int getId();
	int getGrpId();
	int getIconId();
	int getZoom();
	int getEnabled();
	QString getName();
};

class PoiDbIcon : public PoiDbResult
{
  public:
	PoiDbIcon(SQLiteStatement *s);

	int getId();
	int getWidth();
	int getHeight();
	QByteArray getIconData();
};

#endif // __CENTERMARKER_H__
