/*
  poi_icon_dialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "poi_icon_dialog.h"
#include <qstring.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qsizepolicy.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qfileinfo.h>
#include <qcombobox.h>

#include "pos.h"
#include "filedialog.h"
#include "poi_icon.h"
#include "poi_db.h"

#define DIALOG_WIDTH  400

PoiIconDialog::PoiIconDialog( QWidget* parent, const char* name,
								bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
	if (g_machine.resolution() == QVGA) {
		resize(DIALOG_WIDTH / 2, 1);
	} else {
		resize(DIALOG_WIDTH, 1);
	}
	resize(size().expandedTo(minimumSizeHint()));

	setCaption(QString::fromUtf8("POI アイコン追加"));

//	setFont(QFont("lcfont", 16));

	QVBoxLayout *vbox = new QVBoxLayout(this);

	QHBox *file_hbox = new QHBox(this);
	new QLabel(QString::fromUtf8("Path: "), file_hbox);
	file_edit = new QLineEdit(file_hbox);
	QPushButton *file_button = new QPushButton(QString::fromUtf8("参照"), file_hbox);
	vbox->addWidget(file_hbox);

	QHBox *preview_hbox = new QHBox(this);
	new QLabel(QString::fromUtf8("Preview: "), preview_hbox);
	icon_label = new QLabel(preview_hbox);
	icon_label->setFixedSize(16 + 2, 16 + 2);
	icon_label->setFrameShape(QFrame::Panel);
	icon_label->setFrameShadow(QFrame::Sunken);
	icon_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	QLabel *dummy = new QLabel(preview_hbox);
	dummy->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
	QPushButton *preview_button =
		new QPushButton(QString::fromUtf8("Preview"), preview_hbox);
	vbox->addWidget(preview_hbox);

	connect(file_button, SIGNAL(clicked()), this, SLOT(press_file_browse()));
	connect(preview_button, SIGNAL(clicked()), this, SLOT(press_preview()));

#ifndef QWS
	QHBox *op_hbox = new QHBox(this);
	QPushButton *cancel = new QPushButton("Cancel", op_hbox);
	QPushButton *ok = new QPushButton("OK", op_hbox);
	ok->setDefault(true);
	vbox->addWidget(op_hbox);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(reject()));
	connect(ok, SIGNAL(clicked()),
			this, SLOT(accept()));
#endif

	show();
	updateGeometry();
	QSize size = (g_machine.screensize() - this->size()) / 2;
	move(size.width(), size.height());
}

PoiIconDialog::~PoiIconDialog()
{
}

int PoiIconDialog::getIconId()
{
	return icon_id;
}

void PoiIconDialog::accept()
{
	PoiIcon poiicon;

	if (file_edit->text().length() > 0 &&
		!poiicon.load(file_edit->text())) {
		QMessageBox::critical(this,
							  "Error!!",
							  QString::fromUtf8("アイコンファイルが開けません"));
		return;
	}

	PoiDb db(g_db_poi);

	QByteArray *png = poiicon.getPNGdata();
	icon_id = db.addIcon(*png);
	delete png;
	if (icon_id < 0) {
		QMessageBox::critical(this,
							  "Error!!",
							  QString::fromUtf8("DBエラーです"));
		return;
	}

	QDialog::accept();
}

void PoiIconDialog::press_file_browse()
{
	FileDialog dlg(this, QString::fromUtf8("アイコンファイル"));
	dlg.setFileName(file_edit->text());
	if (dlg.exec() == QDialog::Accepted) {
		file_edit->setText(dlg.getFileName());
		press_preview();
	}
}

void PoiIconDialog::press_preview()
{
	PoiIcon poiicon;

	if (!poiicon.load(file_edit->text())) {
		QMessageBox::critical(this,
							  "Error!!",
							  QString::fromUtf8("ファイルが開けません"));
		return;
	}

	QPixmap icon(poiicon.getPixmap());

	if (icon.width() > 16 || icon.height() > 16) {
		QMessageBox::critical(this,
							  "Error!!",
							  QString::fromUtf8("画像サイズが大きすぎます"));
		return;
	}

	icon_label->setPixmap(icon);
	icon_label->repaint();
}
