/*
  poi_search_dialog.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __POI_SEARCH_DIALOG_H__
#define __POI_SEARCH_DIALOG_H__

#include <qvariant.h>
#include <qdialog.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qlistview.h>
#include <qcstring.h>
#include <qdatetime.h>
#include <qmap.h>
#include "pos.h"

class PoiDb;
class ProgressDialog;
class QLineEdit;
class QComboBox;

#define MAX_RESULT 200

class PoiSearchDialog : public QDialog
{
    Q_OBJECT

  public:
    PoiSearchDialog(QWidget* parent = 0, const Pos &center = 0,
					const char* name = 0, bool modal = TRUE, WFlags fl = 0 );
    ~PoiSearchDialog();

	void updateProgress(int now, int max);
	Pos getPos();

  private:

	class PoiResultItem : public QListViewItem
	{
	  public:
		PoiResultItem(QListView * parent);

		QString key(int column, bool ascending) const;
		void setDistance(int distance);
	  private:
		QString distance;
	};

	struct poi_file_info {
		int enable;
		int id;
	};

	QListView *filelistview;
	QListView *result;
	PoiDb *db;
	QPixmap check[2];
	QLineEdit *keyword;
	QComboBox *combo;
	QPixmap arrow;

	void updateFileItem();
	void updateResult(QString const &keyword);
	int calcRadius(QString const &keyword, const QArray<int> &ids, int *count);
	QArray<int> createFileIdArray();

//	static void importProgress(int now, int max, void *data);

//	ProgressDialog *progress;
//	QTime progress_timer;

	typedef QMap<QListViewItem *, struct poi_file_info> FileInfo;
	FileInfo file_info;
	typedef QMap<QListViewItem *, int> ResultInfo;
	ResultInfo result_info;

	Pos center;

  protected:
	bool eventFilter(QObject *obj, QEvent *ev);

  public slots:
	void press_search();

	void click_row(QListViewItem *item, const QPoint &point, int column);

  protected slots:
	void accept(void);
};

#endif // __POI_SEARCH_DIALOG_H__
